package io.zipcoder;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.Array;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ItemParser {

    private int numberOfExceptions;

    public String readFromFile(String filePath) {
        try {
            File file = new File(filePath);
            BufferedReader bufferedReader = new BufferedReader(new FileReader(file));
            String stringFile = "";
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                stringFile += line + "\n";
            }
            bufferedReader.close();
            return stringFile.trim();
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public ArrayList<String> parseRawDataIntoStringArray(String rawData) {
        String stringPattern = "##";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern, rawData);
        return response;
    }

    public Item parseStringIntoItem(String rawItem) throws ItemParseException {
        List<String> keyValues = findKeyValuePairsInRawItemData(rawItem);

        String name = findValue(keyValues.get(0));
        double price = Double.valueOf(findValue(keyValues.get(1)));
        String type = findValue(keyValues.get(2));
        String expiration = findValue(keyValues.get(3));

        return new Item(convertZeroTo0(name), price, type, expiration);
    }

    public List<Item> parseRawDataIntoItems(String rawData) {
        List<Item> itemList = new ArrayList<>();

        List<String> items = parseRawDataIntoStringArray(rawData);

        for (String item : items) {
            try {
                itemList.add(parseStringIntoItem(item));
            } catch (ItemParseException e) {
                numberOfExceptions++;
            }
        }
        return itemList;
    }

    public String findValue(String keyValue) throws ItemParseException {
        List<String> keyAndValue = splitStringWithRegexPattern(":", keyValue);

        if (keyAndValue.size() > 1) {
            return convertToLowerCase(keyAndValue.get(1));
        } else {
            throw new ItemParseException();
        }
    }

    public Map<String, Integer> itemCounts(List<Item> items) {
        Map<String, Integer> itemMap = new HashMap<>();

        for (Item item : items) {
            if (!itemMap.containsKey(item.getName())) {
                itemMap.put(item.getName(), 1);
            } else {
                itemMap.put(item.getName(), itemMap.get(item.getName()) + 1);
            }
        }

        return itemMap;
    }

    public Map<Map<String, Double>, Integer> priceCounts(List<Item> items) {
        Map<String, Double> namePrice = new HashMap<>();
        Map<Map<String, Double>, Integer> itemMap = new HashMap<>();

        for (Item item : items) {
            if (!namePrice.containsKey(item.getName())) {
                namePrice.put(item.getName(), item.getPrice());
            }
        }
        // im tired
        return itemMap;
    }

    public void printOutput(Map<String, Integer> itemMap) {
        for (String name : itemMap.keySet()) {
            System.out.println(String.format("name: %7s \t\t seen: %s times", capitalizeFirst(name), itemMap.get(name)));
            System.out.println("============= \t\t =============");
        }
        System.out.println(String.format("\nErrors \t\t\t\t seen: %s times", getNumberOfExceptions()));
    }

    public String convertToLowerCase(String input) {
        String output = "";

        List<String> upperAlpha = splitStringWithRegexPattern("", "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
        List<String> lowerAlpha = splitStringWithRegexPattern("", "abcdefghijklmnopqrstuvwxyz");
        ArrayList<String> stringCharacters = stringToArrayList(input);

        for (int i = 0; i < stringCharacters.size(); i++) {
            for (int j = 0; j < upperAlpha.size(); j++) {
                if (stringCharacters.get(i).equals(upperAlpha.get(j))) {
                    stringCharacters.set(i, lowerAlpha.get(j));
                }
            }
            output += stringCharacters.get(i);
        }
        return output;
    }

    public String convertZeroTo0(String input) {
        String output = "";
        ArrayList<String> stringCharacters = stringToArrayList(input);

        for (int i = 0; i < stringCharacters.size(); i++) {
            if (stringCharacters.get(i).equals("0")) {
                stringCharacters.set(i, "o");
            }
            output += stringCharacters.get(i);
        }
        return output;
    }

    public String capitalizeFirst(String input) {
        String output = "";
        List<String> upperAlpha = splitStringWithRegexPattern("", "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
        List<String> lowerAlpha = splitStringWithRegexPattern("", "abcdefghijklmnopqrstuvwxyz");
        ArrayList<String> stringCharacters = stringToArrayList(input);

        for (int i = 0; i < stringCharacters.size(); i++) {
            for (int j = 0; j < upperAlpha.size(); j++) {
                if (stringCharacters.get(0).equals(lowerAlpha.get(j))) {
                    stringCharacters.set(0, upperAlpha.get(j));
                }
            }
            output += stringCharacters.get(i);
        }
        return output;
    }

    public ArrayList<String> stringToArrayList(String input) {
        ArrayList<String> stringCharacters = new ArrayList<>();

        Pattern pattern = Pattern.compile(".");
        Matcher matcher = pattern.matcher(input);

        while (matcher.find()) {
            stringCharacters.add(matcher.group());
        }

        return stringCharacters;
    }

    public ArrayList<String> findKeyValuePairsInRawItemData(String rawItem) {
        String stringPattern = "[^a-zA-Z0-9\\:\\.\\/]";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern, rawItem);
        return response;
    }

    private ArrayList<String> splitStringWithRegexPattern(String stringPattern, String inputString) {
        return new ArrayList<>(Arrays.asList(inputString.split(stringPattern)));
    }

    public int getNumberOfExceptions() {
        return numberOfExceptions;
    }

    public static void main(String[] args) {
        ItemParser itemParser = new ItemParser();

        String rawData = itemParser.readFromFile("src/main/resources/RawData.txt");

        List<Item> items = itemParser.parseRawDataIntoItems(rawData);

        Map<String, Integer> itemMap = itemParser.itemCounts(items);

        itemParser.printOutput(itemMap);
    }
}
